import unittest
from unittest.mock import patch

from tmc import points
from tmc.utils import load, load_module, reload_module, get_stdout, check_source
from functools import reduce
import os
import textwrap
from random import randint

exercise = 'src.number_of_elements'
function = 'count_matching_elements'

def get_correct(test_case: list, n: int) -> int:
    return reduce((lambda x,y: x + y), test_case).count(n)


@points('1.count_matching_elements')
class NumberOfElementsTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        with patch('builtins.input', side_effect=[AssertionError("Att be om input från användaren förväntades inte")]):
           cls.module = load_module(exercise, 'en')

    def test_0_main_program_ok(self):
        ok, line = check_source(self.module)
        message = """Koden för att testa funktionerna ska placeras inuti
if __name__ == "__main__":
block. Följande rad ska flyttas:
"""
        self.assertTrue(ok, message+line)

    def test_1_function_exists(self):
        try:
            from src.number_of_elements import count_matching_elements
        except:
            self.assertTrue(False, 'Din kod bör innehålla en funktion som heter count_matching_elements(my_matrix: list, element: int)' )
        try:
            from src.number_of_elements import count_matching_elements
            count_matching_elements([[1, 2]], 1)
        except:
            self.assertTrue(False, 'Se till att den funktionen kan anropas som\ncount_matching_elements([[1, 2]], 1)' )

    def test_2_type_of_return_value(self):
        count_matching_elements = load(exercise, function, 'en')
        val = count_matching_elements([[1]], 1)
        self.assertTrue(type(val) == int, f"Funktion {function} returnerar inte värdet av int-typ med input [[1]].")
    
    def test_3_matrices(self):
        test_cases = (([[1,2,3],[2,3,1],[4,5,6]], 2), ([[1,5,5,3],[2,5,2,5],[0,0,2,5]], 5), ([[1,2,3,4],[2,3,4,5],[3,4,6,5]], 6))
        for test_case in test_cases:
            with patch('builtins.input', side_effect=[AssertionError("Att be om input från användaren förväntades inte")]):
                reload_module(self.module)
                output_at_start = get_stdout()
                count_matching_elements = load(exercise, function, 'en')
                
                correct = get_correct(test_case[0], test_case[1])
                test_case2 = test_case[0][:]
            
                try:
                    test_result = count_matching_elements(test_case[0], test_case[1])
                except:
                    self.assertTrue(False, f"Se till att funktionen fungerar när matrisen är {test_case[0]} och elementet är {test_case[1]}")

                self.assertEqual(correct, test_result, f"The result {test_result} does not match with the model solution {correct} when matrice is {test_case[0]} and element is {test_case[1]}")
                self.assertEqual(test_case[0], test_case2, f"Funktionen bör inte ändra den ursprungliga listan. Värdet ska vara {test_case2} men det är {test_case}.")

    def test_4_random(self):
        for i in range(5):
            test_case = []
            length = randint(3,6)
            for j in range(randint(3,6)):
                test_case.append([randint(1,5) for i in range(length)])
            
            val = randint(1,5)

            with patch('builtins.input', side_effect=[AssertionError("Att be om input från användaren förväntades inte")]):
                reload_module(self.module)
                output_at_start = get_stdout()
                count_matching_elements = load(exercise, function, 'en')
                
                correct = get_correct(test_case, val)
                test_case2 = test_case[:]
        
                try:
                    test_result = count_matching_elements(test_case, val)
                except:
                    self.assertTrue(False, f"Se till att funktionen fungerar när matrisen är {test_case} och elementet är {val}")


                self.assertEqual(correct, test_result, f"Resultatet {test_result} matchar inte modelllösningen {correct} när matrisen är {test_case} och elementet är {val}.")
                self.assertEqual(test_case, test_case2, f"Funktionen bör inte ändra den ursprungliga listan. Värdet ska vara {test_case2} men det är {test_case}.")
              
if __name__ == '__main__':
    unittest.main()